//  ascii_check implementation  ----------------------------------------------//

//  Copyright Shintaro Sakahara 2007.
//
//  Distributed under the Boost Software License, Version 1.0.
//  (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#include <sstream>
#include <iomanip>

#include "ascii_check.hpp"

namespace boost
{
  namespace inspect
  {
    ascii_check::ascii_check() : m_files_with_errors(0)
    {
      register_signature( ".c" );
      register_signature( ".cpp" );
      register_signature( ".cxx" );
      register_signature( ".h" );
      register_signature( ".hpp" );
      register_signature( ".hxx" );
      register_signature( ".inc" );
      register_signature( ".ipp" );
      register_signature( ".inl" );
      register_signature( "Jamfile" );
      register_signature( ".py" );
      register_signature( ".sh" );
      register_signature( ".bat" );
      register_signature( ".txt" );
    }
    
    ascii_check::~ascii_check()
    {
      std::cout << "  " << m_files_with_errors
                << " files with unrecognized characters"
                << std::endl;
    }

    const char * ascii_check::name() const
    {
      return "*S*";
    }
    
    const char * ascii_check::desc() const
    {
      return "unrecognized characters in file";
    }

    void ascii_check::inspect(
        const string & library_name,
        const path & full_path,
        const string & contents )
    {
      bool has_error = false;

      string::size_type line = 1;
      string::const_iterator it = contents.begin();

      while ( it != contents.end() )
      {
        unsigned char ch = *it++;

        if (ch == '\n')
        {
          ++line;
        }
        else if (ch >= 0x80)
        {
          std::ostringstream out;

          out << std::setfill('0') << std::uppercase
              << name()
              << " unrecognized character 0x"
              << std::hex << std::setw(2) << static_cast<unsigned int> (ch)
              << " at line "
              << std::dec << std::setw(1) << line;

          error( library_name, full_path, out.str() );
          has_error = true;

          // skipping same errors on the line
          it = std::find(it, contents.end(), '\n');
        }
      }

      if (has_error)
      {
        ++m_files_with_errors;
      }
    }
  } // namespace inspect
} // namespace boost

